@@ -9,29 +9,37 @@
 
 #include <QFile>
 #include <QFileSystemWatcher>
-#include <QTextStream>
-#include <QSharedPointer>
-#include <QRegularExpression>
 #include <QQmlParserStatus>
+#include <QRegularExpression>
+#include <QSharedPointer>
+#include <QTextStream>
 
 class ReadFile : public QObject, public QQmlParserStatus
 {
-Q_OBJECT
-Q_INTERFACES(QQmlParserStatus)
-Q_PROPERTY(QString contents READ contents NOTIFY contentsChanged)
-Q_PROPERTY(QString path READ path WRITE setPath NOTIFY pathChanged)
-Q_PROPERTY(QString filter READ filter WRITE setFilter FINAL)
+    Q_OBJECT
+    Q_INTERFACES(QQmlParserStatus)
+    Q_PROPERTY(QString contents READ contents NOTIFY contentsChanged)
+    Q_PROPERTY(QString path READ path WRITE setPath NOTIFY pathChanged)
+    Q_PROPERTY(QString filter READ filter WRITE setFilter FINAL)
 public:
     ReadFile();
 
-    QString contents() const { return m_contents; }
-    QString path() const { return m_file.fileName(); }
+    QString contents() const
+    {
+        return m_contents;
+    }
+    QString path() const
+    {
+        return m_file.fileName();
+    }
     void setPath(QString path);
 
     QString filter() const;
     void setFilter(const QString &filter);
 
-    void classBegin() override {}
+    void classBegin() override
+    {
+    }
     void componentComplete() override;
 
 Q_SIGNALS:
@@ -41,7 +49,7 @@ Q_SIGNALS:
 private:
     void process();
     void openNow();
-    void processPath(QString& path);
+    void processPath(QString &path);
 
     bool completed = false;
     QFile m_file;
