@@ -6,30 +6,38 @@
 
 // #define QT_QML_DEBUG
 
-#include <QApplication>
+#include "DiscoverObject.h"
+#include "DiscoverVersion.h"
+#include <DiscoverBackendsFactory.h>
 #include <KAboutData>
 #include <KCrash>
 #include <KDBusService>
-#include <KQuickAddons/QtQuickSettings>
 #include <KLocalizedString>
+#include <KQuickAddons/QtQuickSettings>
+#include <KStartupInfo>
+#include <KWindowSystem>
+#include <QApplication>
 #include <QCommandLineParser>
-#include <QWindow>
-#include "DiscoverObject.h"
-#include <DiscoverBackendsFactory.h>
-#include "DiscoverVersion.h"
-#include <QTextStream>
 #include <QStandardPaths>
+#include <QTextStream>
+#include <QWindow>
+
+#include <QProcessEnvironment>
+#include <QX11Info>
 
 typedef QHash<QString, DiscoverObject::CompactMode> StringCompactMode;
-Q_GLOBAL_STATIC_WITH_ARGS(StringCompactMode, s_decodeCompactMode, (StringCompactMode {
-    { QLatin1String("auto"), DiscoverObject::Auto },
-    { QLatin1String("compact"), DiscoverObject::Compact },
-    { QLatin1String("full"), DiscoverObject::Full }
-}))
+Q_GLOBAL_STATIC_WITH_ARGS(StringCompactMode,
+                          s_decodeCompactMode,
+                          (StringCompactMode{
+                              {QLatin1String("auto"), DiscoverObject::Auto},
+                              {QLatin1String("compact"), DiscoverObject::Compact},
+                              {QLatin1String("full"), DiscoverObject::Full},
+                          }))
 
-QCommandLineParser* createParser()
+QCommandLineParser *createParser()
 {
-    QCommandLineParser* parser = new QCommandLineParser;
+    // clang-format off
+    QCommandLineParser *parser = new QCommandLineParser;
     parser->addOption(QCommandLineOption(QStringLiteral("application"), i18n("Directly open the specified application by its appstream:// URI."), QStringLiteral("name")));
     parser->addOption(QCommandLineOption(QStringLiteral("mime"), i18n("Open with a search for programs that can deal with the given mimetype."), QStringLiteral("name")));
     parser->addOption(QCommandLineOption(QStringLiteral("category"), i18n("Display a list of entries with a category."), QStringLiteral("name")));
@@ -42,31 +50,33 @@ QCommandLineParser* createParser()
     parser->addOption(QCommandLineOption(QStringLiteral("feedback"), i18n("Lists the available options for user feedback")));
     parser->addOption(QCommandLineOption(QStringLiteral("test"), QStringLiteral("Test file"), QStringLiteral("file.qml")));
     parser->addPositionalArgument(QStringLiteral("urls"), i18n("Supports appstream: url scheme"));
+    // clang-format on
     DiscoverBackendsFactory::setupCommandLine(parser);
     KAboutData::applicationData().setupCommandLine(parser);
     return parser;
 }
 
-void processArgs(QCommandLineParser* parser, DiscoverObject* mainWindow)
+void processArgs(QCommandLineParser *parser, DiscoverObject *mainWindow)
 {
-    if(parser->isSet(QStringLiteral("application")))
+    if (parser->isSet(QStringLiteral("application")))
         mainWindow->openApplication(QUrl(parser->value(QStringLiteral("application"))));
-    else if(parser->isSet(QStringLiteral("mime")))
+    else if (parser->isSet(QStringLiteral("mime")))
         mainWindow->openMimeType(parser->value(QStringLiteral("mime")));
-    else if(parser->isSet(QStringLiteral("category")))
+    else if (parser->isSet(QStringLiteral("category")))
         mainWindow->openCategory(parser->value(QStringLiteral("category")));
-    else if(parser->isSet(QStringLiteral("mode")))
+    else if (parser->isSet(QStringLiteral("mode")))
         mainWindow->openMode(parser->value(QStringLiteral("mode")));
     else
         mainWindow->openMode(QStringLiteral("Browsing"));
 
-    if(parser->isSet(QStringLiteral("search")))
+    if (parser->isSet(QStringLiteral("search")))
         Q_EMIT mainWindow->openSearch(parser->value(QStringLiteral("search")));
 
-    if(parser->isSet(QStringLiteral("local-filename")))
+    if (parser->isSet(QStringLiteral("local-filename")))
         mainWindow->openLocalPackage(QUrl::fromUserInput(parser->value(QStringLiteral("local-filename")), {}, QUrl::AssumeLocalFile));
 
-    foreach(const QString &arg, parser->positionalArguments()) {
+    const auto positionalArguments = parser->positionalArguments();
+    for (const QString &arg : positionalArguments) {
         const QUrl url = QUrl::fromUserInput(arg, {}, QUrl::AssumeLocalFile);
         if (url.isLocalFile())
             mainWindow->openLocalPackage(url);
@@ -77,7 +87,18 @@ void processArgs(QCommandLineParser* parser, DiscoverObject* mainWindow)
     }
 }
 
-int main(int argc, char** argv)
+static void raiseWindow(QWindow *window)
+{
+    if (QX11Info::isPlatformX11()) {
+        KStartupInfo::setNewStartupId(window, QX11Info::nextStartupId());
+    } else if (KWindowSystem::isPlatformWayland()) {
+        KWindowSystem::setCurrentXdgActivationToken(qEnvironmentVariable("XDG_ACTIVATION_TOKEN"));
+        KWindowSystem::activateWindow(window->winId());
+    }
+    window->raise();
+}
+
+int main(int argc, char **argv)
 {
     // needs to be set before we create the QGuiApplication
     QCoreApplication::setAttribute(Qt::AA_DisableSessionManager, true);
@@ -89,17 +110,27 @@ int main(int argc, char** argv)
     KCrash::initialize();
     KQuickAddons::QtQuickSettings::init();
     KLocalizedString::setApplicationDomain("plasma-discover");
-    KAboutData about(QStringLiteral("discover"), i18n("Discover"), version, i18n("An application explorer"),
-                     KAboutLicense::GPL, i18n("© 2010-2020 Plasma Development Team"));
-    about.addAuthor(i18n("Aleix Pol Gonzalez"), QString(), QStringLiteral("aleixpol@kde.org"));
-    about.addAuthor(i18n("Nate Graham"), i18n("Quality Assurance, Design and Usability"), QStringLiteral("nate@kde.org"));
-    about.addAuthor(i18n("Dan Leinir Turthra Jensen"), i18n("KNewStuff"), QStringLiteral("admin@leinir.dk"));
+    KAboutData about(QStringLiteral("discover"),
+                     i18n("Discover"),
+                     version,
+                     i18n("An application explorer"),
+                     KAboutLicense::GPL,
+                     i18n("© 2010-2022 Plasma Development Team"));
+    about.addAuthor(i18n("Aleix Pol Gonzalez"), QString(), QStringLiteral("aleixpol@kde.org"), QStringLiteral("https://proli.net"), QStringLiteral("apol"));
+    about.addAuthor(i18n("Nate Graham"),
+                    i18n("Quality Assurance, Design and Usability"),
+                    QStringLiteral("nate@kde.org"),
+                    QStringLiteral("https://pointieststick.com/"),
+                    QStringLiteral("ngraham"));
+    about.addAuthor(i18n("Dan Leinir Turthra Jensen"),
+                    i18n("KNewStuff"),
+                    QStringLiteral("admin@leinir.dk"),
+                    QStringLiteral("https://leinir.dk/"),
+                    QStringLiteral("leinir"));
     about.setProductName("discover/discover");
     about.setProgramLogo(app.windowIcon());
 
-    about.setTranslator(
-            i18ndc(nullptr, "NAME OF TRANSLATORS", "Your names"),
-            i18ndc(nullptr, "EMAIL OF TRANSLATORS", "Your emails"));
+    about.setTranslator(i18ndc(nullptr, "NAME OF TRANSLATORS", "Your names"), i18ndc(nullptr, "EMAIL OF TRANSLATORS", "Your emails"));
 
     KAboutData::setApplicationData(about);
 
@@ -109,11 +140,13 @@ int main(int argc, char** argv)
         parser->process(app);
         about.processCommandLine(parser.data());
         DiscoverBackendsFactory::processCommandLine(parser.data(), parser->isSet(QStringLiteral("test")));
+        const bool feedback = parser->isSet(QStringLiteral("feedback"));
 
-        if(parser->isSet(QStringLiteral("listbackends"))) {
+        if (parser->isSet(QStringLiteral("listbackends"))) {
             QTextStream(stdout) << i18n("Available backends:\n");
             DiscoverBackendsFactory f;
-            foreach(const QString& name, f.allBackendNames(false, true))
+            const auto backendNames = f.allBackendNames(false, true);
+            for (const QString &name : backendNames)
                 QTextStream(stdout) << " * " << name << '\n';
             return 0;
         }
@@ -122,7 +155,7 @@ int main(int argc, char** argv)
             QStandardPaths::setTestModeEnabled(true);
         }
 
-        KDBusService* service = new KDBusService(KDBusService::Unique, &app);
+        KDBusService *service = !feedback ? new KDBusService(KDBusService::Unique, &app) : nullptr;
 
         {
             auto options = parser->optionNames();
@@ -131,37 +164,48 @@ int main(int argc, char** argv)
             QVariantMap initialProperties;
             if (!options.isEmpty() || !parser->positionalArguments().isEmpty())
                 initialProperties = {{QStringLiteral("currentTopLevel"), QStringLiteral("qrc:/qml/LoadingPage.qml")}};
+            if (feedback) {
+                initialProperties.insert("visible", false);
+            }
             mainWindow = new DiscoverObject(s_decodeCompactMode->value(parser->value(QStringLiteral("compact")), DiscoverObject::Full), initialProperties);
         }
-        QObject::connect(&app, &QCoreApplication::aboutToQuit, mainWindow, &DiscoverObject::deleteLater);
-        QObject::connect(service, &KDBusService::activateRequested, mainWindow, [mainWindow](const QStringList &arguments, const QString &/*workingDirectory*/){
-            if (!mainWindow->rootObject())
-                QCoreApplication::instance()->quit();
+        if (feedback) {
+            QTextStream(stdout) << mainWindow->describeSources() << '\n';
+            delete mainWindow;
+            return 0;
+        } else {
+            auto onActivateRequested = [mainWindow](const QStringList &arguments, const QString & /*workingDirectory*/) {
+                auto window = qobject_cast<QWindow *>(mainWindow->rootObject());
+                if (!window) {
+                    // Should never happen anyway
+                    QCoreApplication::instance()->quit();
+                    return;
+                }
 
-            mainWindow->rootObject()->raise();
-            if (arguments.isEmpty())
-                return;
-            QScopedPointer<QCommandLineParser> parser(createParser());
-            parser->parse(arguments);
-            processArgs(parser.data(), mainWindow);
-        });
+                raiseWindow(window);
+
+                if (arguments.isEmpty())
+                    return;
+                QScopedPointer<QCommandLineParser> parser(createParser());
+                parser->parse(arguments);
+                processArgs(parser.data(), mainWindow);
+            };
+            QObject::connect(service, &KDBusService::activateRequested, mainWindow, onActivateRequested);
+        }
+
+        QObject::connect(&app, &QCoreApplication::aboutToQuit, mainWindow, &DiscoverObject::deleteLater);
 
         processArgs(parser.data(), mainWindow);
 
-        if(parser->isSet(QStringLiteral("listmodes"))) {
+        if (parser->isSet(QStringLiteral("listmodes"))) {
             QTextStream(stdout) << i18n("Available modes:\n");
-            foreach(const QString& mode, mainWindow->modes())
+            const auto modes = mainWindow->modes();
+            for (const QString &mode : modes)
                 QTextStream(stdout) << " * " << mode << '\n';
             delete mainWindow;
             return 0;
         }
 
-        if(parser->isSet(QStringLiteral("feedback"))) {
-            QTextStream(stdout) << mainWindow->describeSources() << '\n';
-            delete mainWindow;
-            return 0;
-        }
-
         if (parser->isSet(QStringLiteral("test"))) {
             const QUrl testFile = QUrl::fromUserInput(parser->value(QStringLiteral("test")), {}, QUrl::AssumeLocalFile);
             Q_ASSERT(!testFile.isEmpty() && testFile.isLocalFile());
