@@ -8,26 +8,16 @@
 #define FLATPAKFETCHDATAJOB_H
 
 #include <QByteArray>
-extern "C" {
-#include <flatpak.h>
+#include "flatpak-helper.h"
 #include <glib.h>
-}
 
 class FlatpakResource;
 
 namespace FlatpakRunnables
 {
-    struct SizeInformation {
-        bool valid = false;
-        guint64 downloadSize;
-        guint64 installedSize;
-    };
+FlatpakRemoteRef *findRemoteRef(FlatpakResource *app, GCancellable *cancellable);
 
-    SizeInformation fetchFlatpakSize(FlatpakInstallation *installation, FlatpakResource *app);
-
-    QByteArray fetchMetadata(FlatpakInstallation *installation, FlatpakResource *app);
+QByteArray fetchMetadata(FlatpakResource *app, GCancellable *cancellable);
 }
 
 #endif // FLATPAKFETCHDATAJOB_H
-
-
