@@ -5,43 +5,87 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-
 #include "FlatpakSourcesBackend.h"
-#include "FlatpakResource.h"
 #include "FlatpakBackend.h"
+#include "FlatpakResource.h"
+#include <KConfigGroup>
 #include <KLocalizedString>
 #include <KSharedConfig>
-#include <KConfigGroup>
 #include <QDebug>
 #include <QNetworkAccessManager>
 #include <QNetworkReply>
-#include <QAction>
 
-#include <glib.h>
-#include <QTemporaryFile>
 #include <QStandardPaths>
+#include <QTemporaryFile>
+#include <glib.h>
+#include <resources/DiscoverAction.h>
 #include <resources/StoredResultsStream.h>
 
 class FlatpakSourceItem : public QStandardItem
 {
 public:
-    FlatpakSourceItem(const QString &text) : QStandardItem(text) { }
-    void setFlatpakInstallation(FlatpakInstallation *installation) { m_installation = installation; }
-    FlatpakInstallation *flatpakInstallation() const { return m_installation; }
+    FlatpakSourceItem(const QString &text, FlatpakRemote *remote, FlatpakBackend *backend)
+        : QStandardItem(text)
+        , m_remote(remote)
+        , m_backend(backend)
+    {
+        g_object_ref(remote);
+    }
+    ~FlatpakSourceItem()
+    {
+        g_object_unref(m_remote);
+    }
+
+    void setFlatpakInstallation(FlatpakInstallation *installation)
+    {
+        m_installation = installation;
+    }
+
+    FlatpakInstallation *flatpakInstallation() const
+    {
+        return m_installation;
+    }
+
+    void setData(const QVariant &value, int role) override
+    {
+        // We check isCheckable() so the initial setting of the item doesn't trigger a change
+        if (role == Qt::CheckStateRole && isCheckable()) {
+            const bool disabled = flatpak_remote_get_disabled(m_remote);
+            const bool requestedDisabled = Qt::Unchecked == value;
+            if (disabled != requestedDisabled) {
+                flatpak_remote_set_disabled(m_remote, requestedDisabled);
+                g_autoptr(GError) error = nullptr;
+                if (!flatpak_installation_modify_remote(m_installation, m_remote, nullptr, &error)) {
+                    qWarning() << "set disabled failed" << error->message;
+                    return;
+                }
+
+                if (requestedDisabled) {
+                    m_backend->unloadRemote(m_installation, m_remote);
+                } else {
+                    m_backend->loadRemote(m_installation, m_remote);
+                }
+            }
+        }
+        QStandardItem::setData(value, role);
+    }
 
 private:
-    FlatpakInstallation *m_installation;
+    FlatpakInstallation *m_installation = nullptr;
+    FlatpakRemote *const m_remote;
+    FlatpakBackend *const m_backend;
 };
 
-FlatpakSourcesBackend::FlatpakSourcesBackend(const QVector<FlatpakInstallation *> &installations, AbstractResourcesBackend * parent)
+FlatpakSourcesBackend::FlatpakSourcesBackend(const QVector<FlatpakInstallation *> &installations, AbstractResourcesBackend *parent)
     : AbstractSourcesBackend(parent)
     , m_preferredInstallation(installations.constFirst())
     , m_sources(new QStandardItemModel(this))
-    , m_flathubAction(new QAction(i18n("Add Flathub"), this))
+    , m_flathubAction(new DiscoverAction("flatpak-discover", i18n("Add Flathub"), this))
     , m_noSourcesItem(new QStandardItem(QStringLiteral("-")))
 {
-    m_flathubAction->setToolTip(QStringLiteral("flathub"));
-    connect(m_flathubAction, &QAction::triggered, this, [this](){
+    m_flathubAction->setObjectName(QStringLiteral("flathub"));
+    m_flathubAction->setToolTip(i18n("Makes it possible to easily install the applications listed in https://flathub.org"));
+    connect(m_flathubAction, &DiscoverAction::triggered, this, [this]() {
         addSource(QStringLiteral("https://flathub.org/repo/flathub.flatpakrepo"));
     });
     for (auto installation : installations) {
@@ -59,7 +103,7 @@ FlatpakSourcesBackend::FlatpakSourcesBackend(const QVector<FlatpakInstallation *
 FlatpakSourcesBackend::~FlatpakSourcesBackend()
 {
     QStringList ids;
-    for (int i = 0, c = m_sources->rowCount(); i<c; ++i) {
+    for (int i = 0, c = m_sources->rowCount(); i < c; ++i) {
         auto it = m_sources->item(i);
         ids << it->data(IdRole).toString();
     }
@@ -67,22 +111,25 @@ FlatpakSourcesBackend::~FlatpakSourcesBackend()
     auto conf = KSharedConfig::openConfig();
     KConfigGroup group = conf->group("FlatpakSources");
     group.writeEntry("Sources", ids);
+
+    if (!m_noSourcesItem->model())
+        delete m_noSourcesItem;
 }
 
-QAbstractItemModel* FlatpakSourcesBackend::sources()
+QAbstractItemModel *FlatpakSourcesBackend::sources()
 {
     return m_sources;
 }
 
 bool FlatpakSourcesBackend::addSource(const QString &id)
 {
-    FlatpakBackend* backend = qobject_cast<FlatpakBackend*>(parent());
+    FlatpakBackend *backend = qobject_cast<FlatpakBackend *>(parent());
     const QUrl flatpakrepoUrl(id);
 
     if (id.isEmpty() || !flatpakrepoUrl.isValid())
         return false;
 
-    auto addSource = [=](AbstractResource* res) {
+    auto addSource = [=](AbstractResource *res) {
         if (res)
             backend->installApplication(res);
         else
@@ -90,11 +137,15 @@ bool FlatpakSourcesBackend::addSource(const QString &id)
     };
 
     if (flatpakrepoUrl.isLocalFile()) {
-        addSource(backend->addSourceFromFlatpakRepo(flatpakrepoUrl));
+        auto stream = new ResultsStream(QStringLiteral("FlatpakSource-") + flatpakrepoUrl.toDisplayString());
+        backend->addSourceFromFlatpakRepo(flatpakrepoUrl, stream);
+        connect(stream, &ResultsStream::resourcesFound, this, [addSource](const QVector<AbstractResource *> &res) {
+            addSource(res.constFirst());
+        });
     } else {
         AbstractResourcesBackend::Filters filter;
         filter.resourceUrl = flatpakrepoUrl;
-        auto stream = new StoredResultsStream ({backend->search(filter)});
+        auto stream = new StoredResultsStream({backend->search(filter)});
         connect(stream, &StoredResultsStream::finished, this, [addSource, stream]() {
             const auto res = stream->resources();
             addSource(res.value(0));
@@ -103,10 +154,10 @@ bool FlatpakSourcesBackend::addSource(const QString &id)
     return true;
 }
 
-QStandardItem * FlatpakSourcesBackend::sourceById(const QString& id) const
+QStandardItem *FlatpakSourcesBackend::sourceById(const QString &id) const
 {
-    QStandardItem* sourceIt = nullptr;
-    for (int i = 0, c = m_sources->rowCount(); i<c; ++i) {
+    QStandardItem *sourceIt = nullptr;
+    for (int i = 0, c = m_sources->rowCount(); i < c; ++i) {
         auto it = m_sources->item(i);
         if (it->data(IdRole) == id) {
             sourceIt = it;
@@ -116,12 +167,12 @@ QStandardItem * FlatpakSourcesBackend::sourceById(const QString& id) const
     return sourceIt;
 }
 
-QStandardItem * FlatpakSourcesBackend::sourceByUrl(const QString& _url) const
+QStandardItem *FlatpakSourcesBackend::sourceByUrl(const QString &_url) const
 {
     QUrl url(_url);
 
-    QStandardItem* sourceIt = nullptr;
-    for (int i = 0, c = m_sources->rowCount(); i<c && !sourceIt; ++i) {
+    QStandardItem *sourceIt = nullptr;
+    for (int i = 0, c = m_sources->rowCount(); i < c && !sourceIt; ++i) {
         auto it = m_sources->item(i);
         if (url.matches(it->data(Qt::StatusTipRole).toUrl(), QUrl::StripTrailingSlash)) {
             sourceIt = it;
@@ -135,7 +186,7 @@ bool FlatpakSourcesBackend::removeSource(const QString &id)
 {
     auto sourceIt = sourceById(id);
     if (sourceIt) {
-        FlatpakSourceItem *sourceItem = static_cast<FlatpakSourceItem*>(sourceIt);
+        FlatpakSourceItem *sourceItem = static_cast<FlatpakSourceItem *>(sourceIt);
         g_autoptr(GCancellable) cancellable = g_cancellable_new();
         g_autoptr(GError) error = nullptr;
         const auto installation = sourceItem->flatpakInstallation();
@@ -146,12 +197,18 @@ bool FlatpakSourcesBackend::removeSource(const QString &id)
             toRemoveHash.reserve(refs->len);
             QStringList toRemoveRefs;
             toRemoveRefs.reserve(refs->len);
-            FlatpakBackend* backend = qobject_cast<FlatpakBackend*>(parent());
+            FlatpakBackend *backend = qobject_cast<FlatpakBackend *>(parent());
             for (uint i = 0; i < refs->len; i++) {
-                FlatpakRef *ref= FLATPAK_REF(g_ptr_array_index(refs, i));
+                FlatpakRef *ref = FLATPAK_REF(g_ptr_array_index(refs, i));
 
                 g_autoptr(GError) error = nullptr;
-                FlatpakInstalledRef* installedRef = flatpak_installation_get_installed_ref(installation, flatpak_ref_get_kind(ref), flatpak_ref_get_name(ref), flatpak_ref_get_arch(ref), flatpak_ref_get_branch(ref), cancellable, &error);
+                FlatpakInstalledRef *installedRef = flatpak_installation_get_installed_ref(installation,
+                                                                                           flatpak_ref_get_kind(ref),
+                                                                                           flatpak_ref_get_name(ref),
+                                                                                           flatpak_ref_get_arch(ref),
+                                                                                           flatpak_ref_get_branch(ref),
+                                                                                           cancellable,
+                                                                                           &error);
                 if (installedRef) {
                     auto res = backend->getAppForInstalledRef(installation, installedRef);
                     const auto name = QString::fromUtf8(flatpak_ref_get_name(ref));
@@ -180,7 +237,7 @@ bool FlatpakSourcesBackend::removeSource(const QString &id)
                     g_autoptr(GError) localError = nullptr;
                     g_autoptr(GCancellable) cancellable = g_cancellable_new();
                     g_autoptr(FlatpakTransaction) transaction = flatpak_transaction_new_for_installation(installation, cancellable, &localError);
-                    for (const QString& instRef : qAsConst(toRemoveRefs)) {
+                    for (const QString &instRef : qAsConst(toRemoveRefs)) {
                         const QByteArray refString = instRef.toUtf8();
                         flatpak_transaction_add_uninstall(transaction, refString.constData(), &localError);
                         if (localError)
@@ -192,15 +249,17 @@ bool FlatpakSourcesBackend::removeSource(const QString &id)
                     }
                 });
 
-
-                Q_EMIT proceedRequest(i18n("Removing '%1'", id), i18n("To remove this repository, the following applications must be uninstalled:<ul><li>%1</li></ul>", toRemove.join(QStringLiteral("</li><li>"))));
+                Q_EMIT proceedRequest(i18n("Removing '%1'", id),
+                                      i18n("To remove this repository, the following applications must be uninstalled:<ul><li>%1</li></ul>",
+                                           toRemove.join(QStringLiteral("</li><li>"))));
                 return false;
             }
         } else {
             qWarning() << "could not list refs in repo" << id << error->message;
         }
 
-        if (flatpak_installation_remove_remote(installation, id.toUtf8().constData(), cancellable, &error)) {
+        g_autoptr(GError) errorRemoveRemote = nullptr;
+        if (flatpak_installation_remove_remote(installation, id.toUtf8().constData(), cancellable, &errorRemoveRemote)) {
             m_sources->removeRow(sourceItem->row());
 
             if (m_sources->rowCount() == 0) {
@@ -208,7 +267,7 @@ bool FlatpakSourcesBackend::removeSource(const QString &id)
             }
             return true;
         } else {
-            Q_EMIT passiveMessage(i18n("Failed to remove %1 remote repository: %2", id, QString::fromUtf8(error->message)));
+            Q_EMIT passiveMessage(i18n("Failed to remove %1 remote repository: %2", id, QString::fromUtf8(errorRemoveRemote->message)));
             return false;
         }
     } else {
@@ -221,10 +280,10 @@ bool FlatpakSourcesBackend::removeSource(const QString &id)
 
 QVariantList FlatpakSourcesBackend::actions() const
 {
-    return { QVariant::fromValue<QObject*>(m_flathubAction) };
+    return {QVariant::fromValue<QObject *>(m_flathubAction)};
 }
 
-bool FlatpakSourcesBackend::listRepositories(FlatpakInstallation* installation)
+bool FlatpakSourcesBackend::listRepositories(FlatpakInstallation *installation)
 {
     Q_ASSERT(installation);
 
@@ -248,22 +307,12 @@ bool FlatpakSourcesBackend::listRepositories(FlatpakInstallation* installation)
     return true;
 }
 
-FlatpakRemote * FlatpakSourcesBackend::installSource(FlatpakResource *resource)
+void FlatpakSourcesBackend::populateRemote(FlatpakRemote *remote, const QString &name, const QString &url, const QString &gpgKey)
 {
-    g_autoptr(GCancellable) cancellable = g_cancellable_new();
-
-    auto remote = flatpak_installation_get_remote_by_name(m_preferredInstallation, resource->flatpakName().toUtf8().constData(), cancellable, nullptr);
-    if (remote) {
-        qWarning() << "Source " << resource->flatpakName() << " already exists in" << flatpak_installation_get_path(m_preferredInstallation);
-        return nullptr;
-    }
-
-    remote = flatpak_remote_new(resource->flatpakName().toUtf8().constData());
-    flatpak_remote_set_url(remote, resource->getMetadata(QStringLiteral("repo-url")).toString().toUtf8().constData());
+    flatpak_remote_set_url(remote, url.toUtf8().constData());
     flatpak_remote_set_noenumerate(remote, false);
-    flatpak_remote_set_title(remote, resource->comment().toUtf8().constData());
+    flatpak_remote_set_title(remote, name.toUtf8().constData());
 
-    const QString gpgKey = resource->getMetadata(QStringLiteral("gpg-key")).toString();
     if (!gpgKey.isEmpty()) {
         gsize dataLen = 0;
         g_autofree guchar *data = nullptr;
@@ -275,13 +324,30 @@ FlatpakRemote * FlatpakSourcesBackend::installSource(FlatpakResource *resource)
     } else {
         flatpak_remote_set_gpg_verify(remote, false);
     }
+}
 
+FlatpakRemote *FlatpakSourcesBackend::installSource(FlatpakResource *resource)
+{
+    g_autoptr(GCancellable) cancellable = g_cancellable_new();
+
+    auto remote = flatpak_installation_get_remote_by_name(m_preferredInstallation, resource->flatpakName().toUtf8().constData(), cancellable, nullptr);
+    if (remote) {
+        qWarning() << "Source " << resource->flatpakName() << " already exists in" << flatpak_installation_get_path(m_preferredInstallation);
+        return nullptr;
+    }
+
+    remote = flatpak_remote_new(resource->flatpakName().toUtf8().constData());
+    populateRemote(remote,
+                   resource->comment(),
+                   resource->getMetadata(QStringLiteral("repo-url")).toString(),
+                   resource->getMetadata(QStringLiteral("gpg-key")).toString());
     if (!resource->branch().isEmpty()) {
         flatpak_remote_set_default_branch(remote, resource->branch().toUtf8().constData());
     }
 
-    if (!flatpak_installation_modify_remote(m_preferredInstallation, remote, cancellable, nullptr)) {
-        qWarning() << "Failed to add source " << resource->flatpakName();
+    g_autoptr(GError) error = nullptr;
+    if (!flatpak_installation_modify_remote(m_preferredInstallation, remote, cancellable, &error)) {
+        qWarning() << "Failed to add source " << resource->flatpakName() << error->message;
         return nullptr;
     }
 
@@ -297,21 +363,29 @@ void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation
     const QUrl remoteUrl(QString::fromUtf8(flatpak_remote_get_url(remote)));
 
     const auto theActions = actions();
-    for(const QVariant& act: theActions) {
-        QAction* action = qobject_cast<QAction*>(act.value<QObject*>());
-        if (action->toolTip() == id) {
+    for (const QVariant &act : theActions) {
+        DiscoverAction *action = qobject_cast<DiscoverAction *>(act.value<QObject *>());
+        if (action->objectName() == id) {
             action->setEnabled(false);
             action->setVisible(false);
         }
     }
 
-    FlatpakSourceItem *it = new FlatpakSourceItem(!title.isEmpty() ? title : id);
+    QString label = !title.isEmpty() ? title : id;
+    if (flatpak_installation_get_is_user(installation)) {
+        label = i18n("%1 (user)", label);
+    }
+
+    FlatpakBackend *backend = qobject_cast<FlatpakBackend *>(parent());
+    FlatpakSourceItem *it = new FlatpakSourceItem(label, remote, backend);
     it->setData(remoteUrl.isLocalFile() ? remoteUrl.toLocalFile() : remoteUrl.host(), Qt::ToolTipRole);
     it->setData(remoteUrl, Qt::StatusTipRole);
     it->setData(id, IdRole);
-#if FLATPAK_CHECK_VERSION(1,4,0)
+    it->setCheckState(flatpak_remote_get_disabled(remote) ? Qt::Unchecked : Qt::Checked);
+#if FLATPAK_CHECK_VERSION(1, 4, 0)
     it->setData(QString::fromUtf8(flatpak_remote_get_icon(remote)), IconUrlRole);
 #endif
+    it->setCheckable(true);
     it->setFlatpakInstallation(installation);
 
     int idx = -1;
@@ -321,11 +395,11 @@ void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation
         const auto ids = group.readEntry<QStringList>("Sources", QStringList());
 
         const auto ourIdx = ids.indexOf(id);
-        if (ourIdx<0) { //If not present, we put it on top
+        if (ourIdx < 0) { // If not present, we put it on top
             idx = 0;
         } else {
-            idx=0;
-            for(int c=m_sources->rowCount(); idx<c; ++idx) {
+            idx = 0;
+            for (int c = m_sources->rowCount(); idx < c; ++idx) {
                 const auto compIt = m_sources->item(idx);
                 const int compIdx = ids.indexOf(compIt->data(IdRole).toString());
                 if (compIdx >= ourIdx) {
@@ -350,7 +424,7 @@ QString FlatpakSourcesBackend::idDescription()
     return i18n("Enter a Flatpak repository URI (*.flatpakrepo):");
 }
 
-bool FlatpakSourcesBackend::moveSource(const QString& sourceId, int delta)
+bool FlatpakSourcesBackend::moveSource(const QString &sourceId, int delta)
 {
     auto item = sourceById(sourceId);
     if (!item)
@@ -359,7 +433,7 @@ bool FlatpakSourcesBackend::moveSource(const QString& sourceId, int delta)
     auto prevRow = m_sources->takeRow(row);
     Q_ASSERT(!prevRow.isEmpty());
 
-    const auto destRow = row + (delta>0? delta : delta);
+    const auto destRow = row + (delta > 0 ? delta : delta);
     m_sources->insertRow(destRow, prevRow);
     if (destRow == 0 || row == 0)
         Q_EMIT firstSourceIdChanged();
@@ -368,7 +442,7 @@ bool FlatpakSourcesBackend::moveSource(const QString& sourceId, int delta)
     return true;
 }
 
-int FlatpakSourcesBackend::originIndex(const QString& sourceId) const
+int FlatpakSourcesBackend::originIndex(const QString &sourceId) const
 {
     auto item = sourceById(sourceId);
     return item ? item->row() : INT_MAX;
@@ -383,3 +457,19 @@ void FlatpakSourcesBackend::proceed()
 {
     m_proceedFunctions.pop()();
 }
+
+void FlatpakSourcesBackend::checkRepositories(const QStringList &repoNames)
+{
+    FlatpakBackend *backend = qobject_cast<FlatpakBackend *>(parent());
+    const auto insts = backend->installations();
+    for (const QString &repoName : repoNames) {
+        const QByteArray name = repoName.toUtf8();
+        for (auto installation : insts) {
+            g_autoptr(GError) error = nullptr;
+            auto remote = flatpak_installation_get_remote_by_name(installation, name, nullptr, &error);
+            if (remote) {
+                addRemote(remote, installation);
+            }
+        }
+    }
+}
